#include <pybind11/pybind11.h>
#include <pybind11/numpy.h>
#include <pybind11/stl.h>
#include <vector>
#include <fstream>
#include "stormpci.h"

#define STRINGIFY(x) #x
#define MACRO_STRINGIFY(x) STRINGIFY(x)


typedef StormProcess_tBOARDDATA_t StrikeData;

typedef struct {
    unsigned device_id;
    unsigned experiment_id;
    StrikeData strike;
} BlockBinary;


class BlockReader
{
public:
    BlockReader(const std::string &fn){
        f = std::ifstream(fn, std::fstream::binary);
    }
    BlockBinary& read_block(){
        f.read(reinterpret_cast<char*>(&_single_block), sizeof(BlockBinary));
        if (not f){
            throw;
        } else {
            pos += sizeof(BlockBinary);
        }
        return _single_block;
    }
    std::vector<BlockBinary>& read_all_blocks(){
        bool read_successfully = true;
        while (read_successfully){
            f.read(reinterpret_cast<char*>(&_single_block), sizeof(BlockBinary));
            if (not f){
                read_successfully = false;
            }
            _blocks_vector.push_back(_single_block);
        }
        return _blocks_vector;
    }
    ~BlockReader() {
        f.close();
    };
private:
    int pos = 0;
    BlockBinary _single_block;
    std::ifstream f;
    std::vector<BlockBinary> _blocks_vector;
};

namespace py = pybind11;

PYBIND11_MODULE(stormpci_bin_reader, m) {
    m.doc() = R"pbdoc(
        Read binary blocks from file based on boltek c-structs
        -----------------------

        .. currentmodule:: stormpci_bin_reader

        .. autosummary::
           :toctree: _generate

    )pbdoc";


    py::class_<BlockReader>(m, "BlockReader")
        .def(py::init<const std::string &>())
        .def("read_block", &BlockReader::read_block)
        .def("read_all_blocks", &BlockReader::read_all_blocks);

    py::class_<BlockBinary>(m, "BlockBinary").def(py::init<>())
        .def_readonly("strike", &BlockBinary::strike)
        .def_readonly("device_id", &BlockBinary::device_id)
        .def_readonly("experiment_id", &BlockBinary::experiment_id)
        .def("__repr__",
            [](const BlockBinary &bb) {
                std::stringstream stream;
                stream << "<BlockBinary object with params:\n" <<
                    "\tdevice_id: " << bb.device_id << "\n" <<
                    "\texperiment_id: " << bb.experiment_id << "\n" <<
                    "\tdatetime: " <<
                        (int)bb.strike.lts2_data.year << "-" <<
                        (int)bb.strike.lts2_data.month << "-" <<
                        (int)bb.strike.lts2_data.day << " " <<
                        (int)bb.strike.lts2_data.hours << ":" <<
                        (int)bb.strike.lts2_data.minutes << ":" <<
                        ((float)bb.strike.lts2_data.seconds + (float)bb.strike.lts2_data.TS_time/(float)bb.strike.lts2_data.TS_Osc) << " >";
                return stream.str(); 
            }
        );
        ;

    py::class_<StormProcess_tSATELLITETYPE>(m, "StormProcess_tSATELLITETYPE")
        .def(py::init<>())
        .def_readonly("SVID", &StormProcess_tSATELLITETYPE::SVID)
        .def_readonly("mode", &StormProcess_tSATELLITETYPE::mode)
        .def_readonly("signal_strength", &StormProcess_tSATELLITETYPE::signal_strength)
        .def_readonly("channel_status", &StormProcess_tSATELLITETYPE::channel_status)
        ;

    py::class_<StormProcess_tTIMESTAMPINFO>(m, "StormProcess_tTIMESTAMPINFO")
        .def(py::init<>())
        .def_readonly("TS_valid", &StormProcess_tTIMESTAMPINFO::TS_valid)
        .def_readonly("TS_Osc", &StormProcess_tTIMESTAMPINFO::TS_Osc)
        .def_readonly("TS_time", &StormProcess_tTIMESTAMPINFO::TS_time)
        .def_readonly("TS_10ms", &StormProcess_tTIMESTAMPINFO::TS_10ms)
        .def_readonly("capture_time", &StormProcess_tTIMESTAMPINFO::capture_time)
        .def_readonly("gps_data_valid", &StormProcess_tTIMESTAMPINFO::gps_data_valid)
        .def_readonly("month", &StormProcess_tTIMESTAMPINFO::month)
        .def_readonly("day", &StormProcess_tTIMESTAMPINFO::day)
        .def_readonly("year", &StormProcess_tTIMESTAMPINFO::year)
        .def_readonly("hours", &StormProcess_tTIMESTAMPINFO::hours)
        .def_readonly("minutes", &StormProcess_tTIMESTAMPINFO::minutes)
        .def_readonly("seconds", &StormProcess_tTIMESTAMPINFO::seconds)
        .def_readonly("latitude_mas", &StormProcess_tTIMESTAMPINFO::latitude_mas)
        .def_readonly("longitude_mas", &StormProcess_tTIMESTAMPINFO::longitude_mas)
        .def_readonly("latitude_ns", &StormProcess_tTIMESTAMPINFO::latitude_ns)
        .def_readonly("longitude_ew", &StormProcess_tTIMESTAMPINFO::longitude_ew)
        .def_readonly("height_cm", &StormProcess_tTIMESTAMPINFO::height_cm)
        .def_readonly("dop", &StormProcess_tTIMESTAMPINFO::dop)
        .def_readonly("satellites_visible", &StormProcess_tTIMESTAMPINFO::satellites_visible)
        .def_readonly("satellites_tracked", &StormProcess_tTIMESTAMPINFO::satellites_tracked)
        .def_readonly("receiver_status", &StormProcess_tTIMESTAMPINFO::receiver_status)
        .def_readonly("oscillator_temperature", &StormProcess_tTIMESTAMPINFO::oscillator_temperature)
        .def_readonly("serial_number", &StormProcess_tTIMESTAMPINFO::serial_number)
        .def_property_readonly("satellite", [](const StormProcess_tTIMESTAMPINFO &lts2) {
            return std::vector<StormProcess_tSATELLITETYPE> (lts2.satellite, lts2.satellite + sizeof(lts2.satellite));
        })
        ;

    py::class_<StrikeData>(m, "StrikeData")
        .def(py::init<>())
        .def_property_readonly("EFieldBuf", [](const StrikeData &sdata) {
            return std::vector<double> (sdata.EFieldBuf, sdata.EFieldBuf + BOLTEK_BUFFERSIZE);
        })
        .def_property_readonly("NorthBuf", [](const StrikeData &sdata) {
            return std::vector<double> (sdata.NorthBuf, sdata.NorthBuf + BOLTEK_BUFFERSIZE);
        })
        .def_property_readonly("EastBuf", [](const StrikeData &sdata) {
            return std::vector<double> (sdata.EastBuf, sdata.EastBuf + BOLTEK_BUFFERSIZE);
        })
        .def_readonly("NorthMaxPos", &StrikeData::NorthMaxPos)
        .def_readonly("NorthMinPos", &StrikeData::NorthMinPos)
        .def_readonly("EastMaxPos", &StrikeData::EastMaxPos)
        .def_readonly("EastMinPos", &StrikeData::EastMinPos)
        .def_readonly("North_Pk", &StrikeData::North_Pk)
        .def_readonly("East_Pk", &StrikeData::East_Pk)
        .def_readonly("NorthPol", &StrikeData::NorthPol)
        .def_readonly("EastPol", &StrikeData::EastPol)
        .def_readonly("EFieldPol", &StrikeData::EFieldPol)
        .def_readonly("lts2_data", &StrikeData::lts2_data)
        ;


#ifdef VERSION_INFO
    m.attr("__version__") = MACRO_STRINGIFY(VERSION_INFO);
#else
    m.attr("__version__") = "dev";
#endif
}
